@@ -5,7 +5,7 @@ with open("README.md", "r") as fh:
 
 setuptools.setup(
     name="statinf",
-    version="1.0.12",
+    version="1.0.13",
     author="Florian Felice",
     author_email="florian.felice@outlook.com",
     description="A library for statistics and causal inference",
@@ -18,7 +18,8 @@ setuptools.setup(
           "numpy>=1.16.3",
           "scipy>=1.2.1",
           "theano>=1.0.4",
-          "pycof>=1.0.19"
+          "pycof>=1.0.19",
+          "matplotlib>=3.1.1"
       ],
     classifiers=[
         "Programming Language :: Python :: 3",
